package com.diagbot.facade;

import com.diagbot.enums.RedisEnum;
import com.diagbot.enums.StandConvertEnum;
import com.diagbot.util.RedisUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.stereotype.Component;

/**
 * @Description:
 * @Author:zhoutg
 * @time: 2018/11/23 11:37
 */
@Component
public class CacheFacade {

    @Autowired
    RedisUtil redisUtil;
    @Autowired
    NeoFacade neoFacade;

    /**
     * 清除缓存信息
     *
     * @return
     */
    @CacheEvict(value = "cache", allEntries = true)
    public void clear() {

    }

    /**
     * 清除标准词转换缓存信息
     *
     * @return
     */
    public void clearStandConvert() {
        redisUtil.deleteByPrex(StandConvertEnum.disease.getName());
        redisUtil.deleteByPrex(StandConvertEnum.drug.getName());
        redisUtil.deleteByPrex(StandConvertEnum.lis.getName());
        redisUtil.deleteByPrex(StandConvertEnum.operation.getName());
        redisUtil.deleteByPrex(StandConvertEnum.pacs.getName());
        redisUtil.deleteByPrex(StandConvertEnum.symptom.getName());
        redisUtil.deleteByPrex(StandConvertEnum.vital.getName());
        redisUtil.deleteByPrex(StandConvertEnum.transfusion.getName());
    }

    /**
     * 加载药品类型缓存
     *
     * @return
     */
    public void loadDrugTypeCache() {
        redisUtil.deleteByPrex(RedisEnum.drugType.getName());
        neoFacade.getDrugCache();

    }

    /**
     * 加载数据库中症状特征缓存
     *
     * @return
     */
    public void getSymptomFeatureCache() {
        redisUtil.deleteByPrex(RedisEnum.symptomFeature.getName());
        neoFacade.symptomFeatureCache();
    }

    /**
     * 加载数据库中疾病对应缓存
     *
     * @return
     */
    public void getDiseaseCorrespondCache() {
        redisUtil.deleteByPrex(RedisEnum.diseaseCorrespond.getName());
        neoFacade.diseaseCorrespondCache();
    }

    /**
     * 加载数据库中疾病过滤缓存
     *
     * @return
     */
    public void getdiseaseFilterCache() {
        redisUtil.deleteByPrex(RedisEnum.diseaseFilter.getName());
        neoFacade.diseaseFilterCache();
    }

    /**
     * 缓存医学知识库中的所有规则
     *
     * @return
     */
    public void loadAllRuleCache(){
        redisUtil.deleteByPrex(RedisEnum.allRule.getName());
        neoFacade.allRuleCache();
    }

    public void clearLoadCache(String str) {
        redisUtil.deleteByPrex(str);
    }
}
