package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.dto.DictionaryInfoDTO;
import com.diagbot.dto.KlConceptDetailDTO;
import com.diagbot.dto.KlConceptStaticDTO;
import com.diagbot.dto.StaticKnowledgeDTO;
import com.diagbot.dto.StaticKnowledgeDetailDTO;
import com.diagbot.entity.KlConcept;
import com.diagbot.entity.KlConceptDetail;
import com.diagbot.entity.KlConceptStatic;
import com.diagbot.entity.KlRelation;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.enums.LexiconEnum;
import com.diagbot.enums.StatusEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.service.KlConceptDetailService;
import com.diagbot.service.impl.KlConceptStaticServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.DateUtil;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.vo.ChangeStatusVO;
import com.diagbot.vo.IdVO;
import com.diagbot.vo.KlConceptStaticPageVO;
import com.diagbot.vo.KlConceptStaticVO;
import com.diagbot.vo.StaticKnowledgeVO;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2021/2/23 16:14
 */
@Component
public class KlConceptStaticFacade extends KlConceptStaticServiceImpl {
    @Autowired
    private KlConceptFacade klConceptFacade;
    @Autowired
    private KlRelationFacade klRelationFacade;
    @Autowired
    private KlConceptDetailFacade klConceptDetailFacade;
    @Autowired
    private KlDictionaryInfoFacade klDictionaryInfoFacade;

    @Autowired
    @Qualifier("klConceptDetailServiceImpl")
    private KlConceptDetailService klConceptDetailService;

    /**
     * 获取静态知识
     *
     * @param staticKnowledgeVO
     * @return
     */
    public StaticKnowledgeDTO getStaticKnowledge(StaticKnowledgeVO staticKnowledgeVO) {
        StaticKnowledgeDTO staticKnowledgeDTO = new StaticKnowledgeDTO();
        Integer type = convertType(staticKnowledgeVO.getType(), 1);
        if (type == null) {
            throw new CommonException(CommonErrorCode.PARAM_IS_ERROR, "请输入正确类型（1-诊断、2-药品、3-检验套餐、4-检验细项、5-检查、6-检查子项、7-手术和操作）");
        }
        staticKnowledgeVO.setType(type);

        KlConcept concept = klConceptFacade.getOne(new QueryWrapper<KlConcept>()
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("lib_name", staticKnowledgeVO.getName())
                .eq("lib_type", type), false);
        if (concept == null) {
            throw new CommonException(CommonErrorCode.NOT_EXISTS, "标准词不存在");
        }

        KlConceptStatic staticInfo = this.getOne(new QueryWrapper<KlConceptStatic>()
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("concept_id", concept.getId())
                .eq("status", StatusEnum.Enable.getKey()), false);

        if (staticInfo == null) {
            //检验子项或检查子项向上级取静态知识
            if (type.equals(LexiconEnum.LisSubName.getKey())
                    || type.equals(LexiconEnum.PacsSubName.getKey())) {
                List<KlRelation> relations = klRelationFacade.list(new QueryWrapper<KlRelation>()
                        .eq("is_deleted", IsDeleteEnum.N.getKey())
                        .eq("end_id", concept.getId())
                        .eq("relation_id", 600));
                if (ListUtil.isNotEmpty(relations)) {
                    staticInfo = this.getOne(new QueryWrapper<KlConceptStatic>()
                            .eq("is_deleted", IsDeleteEnum.N.getKey())
                            .in("concept_id", relations.stream().map(i -> i.getStartId()).collect(Collectors.toList()))
                            .eq("status", StatusEnum.Enable.getKey()), false);
                    if (staticInfo == null) {
                        throw new CommonException(CommonErrorCode.NOT_EXISTS, "缺少静态信息");
                    }
                    concept = klConceptFacade.getById(staticInfo.getConceptId());
                } else {
                    throw new CommonException(CommonErrorCode.NOT_EXISTS, "缺少静态信息");
                }
            } else {
                throw new CommonException(CommonErrorCode.NOT_EXISTS, "缺少静态信息");
            }
        }

        //详情信息
        String sql = "";
        if (ListUtil.isNotEmpty(staticKnowledgeVO.getContentTypes())) {

            for (Integer contentType : staticKnowledgeVO.getContentTypes()) {
                if (contentType.equals(0)) {
                    sql = "";
                    break;
                }
                if (StringUtil.isNotBlank(sql)) {
                    sql += " or ";
                }
                sql += "find_in_set(" + contentType + ",content_type)";
            }
            if (StringUtil.isNotBlank(sql)) {
                sql = "(" + sql + ")";
            }
        }
        List<KlConceptDetail> details = klConceptDetailFacade.list(new QueryWrapper<KlConceptDetail>()
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("concept_id", concept.getId())
                .apply(sql)
                .orderByAsc("order_no"));

        List<StaticKnowledgeDetailDTO> detailList
                = BeanUtil.listCopyTo(details, StaticKnowledgeDetailDTO.class);
        List<StaticKnowledgeDetailDTO> introduceList = detailList
                .stream()
                .filter(i -> Arrays.asList(i.getContentType().split(",")).contains("1"))
                .collect(Collectors.toList());
        List<StaticKnowledgeDetailDTO> noticeList = detailList
                .stream()
                .filter(i -> Arrays.asList(i.getContentType().split(",")).contains("2"))
                .collect(Collectors.toList());
        List<StaticKnowledgeDetailDTO> clinicalPathwayList = detailList
                .stream()
                .filter(i -> Arrays.asList(i.getContentType().split(",")).contains("3"))
                .collect(Collectors.toList());
        List<StaticKnowledgeDetailDTO> treatInfoList = detailList
                .stream()
                .filter(i -> Arrays.asList(i.getContentType().split(",")).contains("4"))
                .collect(Collectors.toList());
        Map<String, List<StaticKnowledgeDetailDTO>> detailMap = new HashMap<>();
        if (ListUtil.isEmpty(staticKnowledgeVO.getContentTypes()) || staticKnowledgeVO.getContentTypes().contains(0)) {
            if (ListUtil.isNotEmpty(introduceList)) {
                detailMap.put("静态知识", introduceList);
            }
            if (ListUtil.isNotEmpty(noticeList)) {
                detailMap.put("注意事项", noticeList);
            }
            if (ListUtil.isNotEmpty(clinicalPathwayList)) {
                detailMap.put("临床路径", clinicalPathwayList);
            }
            if (ListUtil.isNotEmpty(treatInfoList)) {
                detailMap.put("治疗方案", treatInfoList);
            }
        } else {
            if (ListUtil.isNotEmpty(introduceList) && staticKnowledgeVO.getContentTypes().contains(1)) {
                detailMap.put("静态知识", introduceList);
            }
            if (ListUtil.isNotEmpty(noticeList) && staticKnowledgeVO.getContentTypes().contains(2)) {
                detailMap.put("注意事项", noticeList);
            }
            if (ListUtil.isNotEmpty(clinicalPathwayList) && staticKnowledgeVO.getContentTypes().contains(3)) {
                detailMap.put("临床路径", clinicalPathwayList);
            }
            if (ListUtil.isNotEmpty(treatInfoList) && staticKnowledgeVO.getContentTypes().contains(4)) {
                detailMap.put("治疗方案", treatInfoList);
            }
        }

        staticKnowledgeDTO.setId(concept.getId());
        staticKnowledgeDTO.setName(concept.getLibName());
        Integer retType = convertType(concept.getLibType(), 2);
        staticKnowledgeDTO.setType(retType == null ? "" : retType.toString());
        staticKnowledgeDTO.setClinicalPathwayName(staticInfo.getClinicalPathwayName());
        staticKnowledgeDTO.setNoticeName(staticInfo.getNoticeName());
        staticKnowledgeDTO.setDetails(detailMap);
        return staticKnowledgeDTO;
    }

    /**
     * 分页查询
     *
     * @param klConceptStaticPageVO
     * @return
     */
    public IPage<KlConceptStaticDTO> getPage(KlConceptStaticPageVO klConceptStaticPageVO) {
        Integer type = convertType(klConceptStaticPageVO.getType(), 1);
        klConceptStaticPageVO.setType(type);

        List<DictionaryInfoDTO> dicTypeName = klDictionaryInfoFacade.getListByGroupType(13);
        Map<String, String> dicTypeNameMap
                = EntityUtil.makeMapWithKeyValue(dicTypeName, "val", "name");

        IPage<KlConceptStaticDTO> page = super.getPage(klConceptStaticPageVO);
        List<KlConceptStaticDTO> records = page.getRecords();
        if (ListUtil.isNotEmpty(records)) {
            records.forEach(record -> {
                String typeName = LexiconEnum.getName(record.getType());
                record.setTypeName(dicTypeNameMap.get(typeName));
                Integer retType = convertType(record.getType(), 2);
                record.setType(retType);
            });
        }
        page.setRecords(records);
        return page;
    }

    /**
     * 保存记录（新增or修改）
     *
     * @param klConceptStaticVO
     * @return
     */
    public Boolean saveOrUpdateRecord(KlConceptStaticVO klConceptStaticVO) {
        //String userId = SysUserUtils.getCurrentPrincipleID();
        if (klConceptStaticVO.getUserId() == null) {
            klConceptStaticVO.setUserId(0L);
        }
        Date now = DateUtil.now();

        Integer type = convertType(klConceptStaticVO.getType(), 1);
        klConceptStaticVO.setType(type);

        KlConcept concept = new KlConcept();
        if (klConceptStaticVO.getId() != null) {
            concept = klConceptFacade.getById(klConceptStaticVO.getId());
        } else {
            concept = klConceptFacade.getOne(new QueryWrapper<KlConcept>()
                    .eq("is_deleted", IsDeleteEnum.N.getKey())
                    .eq("lib_name", klConceptStaticVO.getName())
                    .eq("lib_type", type), false);
        }
        if (concept == null) {
            throw new CommonException(CommonErrorCode.NOT_EXISTS, "标准术语不存在");
        }
        if (!concept.getLibName().equals(klConceptStaticVO.getName())) {
            throw new CommonException(CommonErrorCode.PARAM_IS_ERROR, "标准术语名称不允许修改");
        }

        Long conceptId = concept.getId();
        KlConceptStatic staticInfo = this.getOne(new QueryWrapper<KlConceptStatic>()
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("concept_id", concept.getId()), false);

        if (staticInfo == null) {
            staticInfo = new KlConceptStatic();
            staticInfo.setId(null);
            staticInfo.setConceptId(concept.getId());
            staticInfo.setStatus(StatusEnum.Enable.getKey());
            staticInfo.setCreator(klConceptStaticVO.getUserId().toString());
            staticInfo.setGmtCreate(now);
        }
        staticInfo.setClinicalPathwayName(klConceptStaticVO.getClinicalPathwayName());
        staticInfo.setNoticeName(klConceptStaticVO.getNoticeName());
        staticInfo.setModifier(klConceptStaticVO.getUserId().toString());
        staticInfo.setGmtModified(now);

        List<KlConceptDetail> details = Lists.newLinkedList();
        if (ListUtil.isNotEmpty(klConceptStaticVO.getDetails())) {
            details = BeanUtil.listCopyTo(klConceptStaticVO.getDetails(), KlConceptDetail.class);
        }
        details.forEach(detail -> {
            detail.setConceptId(conceptId);
            detail.setCreator(klConceptStaticVO.getUserId().toString());
            detail.setGmtCreate(now);
            detail.setModifier(klConceptStaticVO.getUserId().toString());
            detail.setGmtModified(now);
        });

        //保存静态知识基础信息
        this.saveOrUpdate(staticInfo);

        //删除已有明细
        klConceptDetailService.remove(new QueryWrapper<KlConceptDetail>()
                .eq("concept_id", concept.getId()));

        //保存新的明细
        klConceptDetailService.saveBatch(details);

        return true;
    }

    /**
     * 启用禁用
     *
     * @param changeStatusVO
     * @return
     */
    public Boolean changeStatus(ChangeStatusVO changeStatusVO) {
        //String userId = SysUserUtils.getCurrentPrincipleID();
        if (changeStatusVO.getUserId() == null) {
            changeStatusVO.setUserId(0L);
        }
        Date now = DateUtil.now();
        //更新主表
        this.update(new UpdateWrapper<KlConceptStatic>()
                .eq("concept_id", changeStatusVO.getId())
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .set("gmt_modified", now)
                .set("modifier", changeStatusVO.getUserId())
                .set("status", changeStatusVO.getStatus()));
        //更新明细表
        klConceptDetailService.update(new UpdateWrapper<KlConceptDetail>()
                .eq("concept_id", changeStatusVO.getId())
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .set("gmt_modified", now)
                .set("modifier", changeStatusVO.getUserId()));
        return true;
    }

    /**
     * 根据术语id获取静态信息
     *
     * @param idVO
     * @return
     */
    public KlConceptStaticDTO getRecordById(IdVO idVO) {
        KlConceptStaticDTO klConceptStaticDTO = new KlConceptStaticDTO();

        List<DictionaryInfoDTO> dicTypeName = klDictionaryInfoFacade.getListByGroupType(13);
        Map<String, String> dicTypeNameMap
                = EntityUtil.makeMapWithKeyValue(dicTypeName, "val", "name");

        KlConcept concept = klConceptFacade.getById(idVO.getId());
        if (concept == null) {
            return null;
        }

        KlConceptStatic staticInfo = this.getOne(new QueryWrapper<KlConceptStatic>()
                .eq("concept_id", idVO.getId()), false);

        if (staticInfo == null) {
            return null;
        }

        List<KlConceptDetail> details = klConceptDetailFacade.list(new QueryWrapper<KlConceptDetail>()
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("concept_id", idVO.getId())
                .orderByAsc("order_no"));

        klConceptStaticDTO.setId(concept.getId());
        klConceptStaticDTO.setName(concept.getLibName());
        klConceptStaticDTO.setType(convertType(concept.getLibType(), 2));
        klConceptStaticDTO.setTypeName(dicTypeNameMap.get(LexiconEnum.getName(concept.getLibType())));

        klConceptStaticDTO.setStatus(staticInfo.getStatus());
        klConceptStaticDTO.setGmtModified(staticInfo.getGmtModified());
        klConceptStaticDTO.setModifier(staticInfo.getModifier());
        klConceptStaticDTO.setClinicalPathwayName(staticInfo.getClinicalPathwayName());
        klConceptStaticDTO.setNoticeName(staticInfo.getNoticeName());

        klConceptStaticDTO.setTitle(ListUtil.isNotEmpty(details)
                ? details.stream().map(i -> i.getTitle()).collect(Collectors.joining("、"))
                : "");
        klConceptStaticDTO.setDetails(ListUtil.isNotEmpty(details)
                ? BeanUtil.listCopyTo(details, KlConceptDetailDTO.class)
                : new ArrayList<>());
        return klConceptStaticDTO;
    }

    /**
     * 是否已存在
     *
     * @param klConceptStaticVO
     * @return
     */
    public Boolean isExist(KlConceptStaticVO klConceptStaticVO) {
        Integer type = convertType(klConceptStaticVO.getType(), 1);
        klConceptStaticVO.setType(type);

        KlConcept concept = new KlConcept();
        if (klConceptStaticVO.getId() != null) {
            concept = klConceptFacade.getById(klConceptStaticVO.getId());
        } else {
            concept = klConceptFacade.getOne(new QueryWrapper<KlConcept>()
                    .eq("is_deleted", IsDeleteEnum.N.getKey())
                    .eq("lib_name", klConceptStaticVO.getName())
                    .eq("lib_type", type), false);
        }
        if (concept == null) {
            throw new CommonException(CommonErrorCode.NOT_EXISTS, "标准术语不存在");
        }
        if (!concept.getLibName().equals(klConceptStaticVO.getName())) {
            throw new CommonException(CommonErrorCode.PARAM_IS_ERROR, "标准术语名称不允许修改");
        }

        KlConceptStatic staticInfo = this.getOne(new QueryWrapper<KlConceptStatic>()
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("concept_id", concept.getId()), false);

        if (staticInfo == null) {
            return false;
        }

        List<KlConceptDetail> details = klConceptDetailFacade.list(new QueryWrapper<KlConceptDetail>()
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("concept_id", concept.getId()));

        if (ListUtil.isEmpty(details)) {
            return false;
        }
        return true;
    }

    /**
     * 页面展示类型转换
     *
     * @param type
     * @param flag
     * @return
     */
    public Integer convertType(Integer type, Integer flag) {
        Integer retType = null;
        if (flag.equals(1)) {
            //1-诊断、2-药品、3-检验套餐、4-检验细项、5-检查、6-检查子项、7-手术和操作
            switch (type) {
                case 1:
                    retType = LexiconEnum.Disease.getKey();
                    break;
                case 2:
                    retType = LexiconEnum.Medicine.getKey();
                    break;
                case 3:
                    retType = LexiconEnum.LisName.getKey();
                    break;
                case 4:
                    retType = LexiconEnum.LisSubName.getKey();
                    break;
                case 5:
                    retType = LexiconEnum.PacsName.getKey();
                    break;
                case 6:
                    retType = LexiconEnum.PacsSubName.getKey();
                    break;
                case 7:
                    retType = LexiconEnum.Operation.getKey();
                    break;
                default:
                    break;
            }
        } else if (flag.equals(2)) {
            if (type.equals(LexiconEnum.Disease.getKey())) {
                retType = 1;
            } else if (type.equals(LexiconEnum.Medicine.getKey())) {
                retType = 2;
            } else if (type.equals(LexiconEnum.LisName.getKey())) {
                retType = 3;
            } else if (type.equals(LexiconEnum.LisSubName.getKey())) {
                retType = 4;
            } else if (type.equals(LexiconEnum.PacsName.getKey())) {
                retType = 5;
            } else if (type.equals(LexiconEnum.PacsSubName.getKey())) {
                retType = 6;
            } else if (type.equals(LexiconEnum.Operation.getKey())) {
                retType = 7;
            }
        }
        return retType;
    }
}