package com.diagbot.facade;

import com.diagbot.dto.DictionaryInfoDTO;
import com.diagbot.dto.DiseaseInfoDTO;
import com.diagbot.dto.DrugInfoDTO;
import com.diagbot.dto.IndexDTO;
import com.diagbot.dto.LisDetailDTO;
import com.diagbot.dto.OperationInfoDTO;
import com.diagbot.dto.RetrievalDTO;
import com.diagbot.dto.StaticKnowledgeIndexDTO;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.vo.MedRetrievalVO;
import com.diagbot.vo.RetrievalVO;
import com.diagbot.vo.StaticKnowledgeIndexVO;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2021/2/22 10:35
 */
@Component
public class MedRetrievalFacade {
    @Autowired
    private KlConceptFacade klConceptFacade;
    @Autowired
    private DictionaryFacade dictionaryFacade;

    /**
     * 检索
     *
     * @param retrievalVO
     * @return
     */
    public RetrievalDTO index(RetrievalVO retrievalVO) {
        if (retrievalVO.getSize() == null) {
            retrievalVO.setSize(100);
        }
        RetrievalDTO retrievalDTO = new RetrievalDTO();
        MedRetrievalVO medRetrievalVO = new MedRetrievalVO();
        BeanUtil.copyProperties(retrievalVO, medRetrievalVO);
        medRetrievalVO.setTypeIds(new ArrayList<>());
        List<IndexDTO> indexList = Lists.newLinkedList();

        /**
         * 类型：1-化验大项、2-化验小项、3-辅检、4-诊断、5-药品、6-手术和操作、7-科室、8-输血、9-症状
         */
        switch (retrievalVO.getType()) {
            case 1:
                medRetrievalVO.setTypeId(107);
                medRetrievalVO.getTypeIds().add(107);
                indexList = klConceptFacade.index(medRetrievalVO);
                if (ListUtil.isNotEmpty(indexList)) {
                    retrievalDTO.setLisNames(indexList.stream().map(i -> i.getName()).collect(Collectors.toList()));
                }
                break;
            case 2:
                medRetrievalVO.setTypeId(108);
                medRetrievalVO.getTypeIds().add(108);
                indexList = klConceptFacade.index(medRetrievalVO);
                if (ListUtil.isNotEmpty(indexList)) {
                    List<LisDetailDTO> lisDetails = Lists.newLinkedList();
                    indexList.forEach(item -> {
                        LisDetailDTO lisDetail = new LisDetailDTO();
                        BeanUtil.copyProperties(item, lisDetail);
                        lisDetail.setName(item.getPackName());
                        lisDetail.setUniqueName(item.getName());
                        //默认套餐名称
                        if (StringUtil.isBlank(lisDetail.getName())) {
                            lisDetail.setName(lisDetail.getUniqueName());
                        }
                        lisDetails.add(lisDetail);
                    });
                    retrievalDTO.setLisDetailNames(lisDetails);
                }
                break;
            case 3:
                medRetrievalVO.setTypeId(109);
                medRetrievalVO.getTypeIds().addAll(Arrays.asList(new Integer[] { 109, 110 }));
                indexList = klConceptFacade.index(medRetrievalVO);
                if (ListUtil.isNotEmpty(indexList)) {
                    retrievalDTO.setPacsNames(indexList.stream().map(i -> i.getName()).collect(Collectors.toList()));
                }
                break;
            case 4:
                medRetrievalVO.setTypeId(100);
                medRetrievalVO.getTypeIds().add(100);
                indexList = klConceptFacade.index(medRetrievalVO);
                if (ListUtil.isNotEmpty(indexList)) {
                    retrievalDTO.setDiseaseNames(BeanUtil.listCopyTo(indexList, DiseaseInfoDTO.class));
                }
                break;
            case 5:
                medRetrievalVO.setTypeId(101);
                medRetrievalVO.getTypeIds().add(101);
                indexList = klConceptFacade.index(medRetrievalVO);
                if (ListUtil.isNotEmpty(indexList)) {
                    retrievalDTO.setDrugNames(BeanUtil.listCopyTo(indexList, DrugInfoDTO.class));
                }
                break;
            case 6:
                medRetrievalVO.setTypeId(106);
                medRetrievalVO.getTypeIds().add(106);
                indexList = klConceptFacade.index(medRetrievalVO);
                if (ListUtil.isNotEmpty(indexList)) {
                    retrievalDTO.setOperationNames(BeanUtil.listCopyTo(indexList, OperationInfoDTO.class));
                }
                break;
            case 7:
                medRetrievalVO.setTypeId(115);
                medRetrievalVO.getTypeIds().add(115);
                indexList = klConceptFacade.index(medRetrievalVO);
                if (ListUtil.isNotEmpty(indexList)) {
                    retrievalDTO.setDeptNames(indexList.stream().map(i -> i.getName()).collect(Collectors.toList()));
                }
                break;
            case 8:
                medRetrievalVO.setTypeId(113);
                medRetrievalVO.getTypeIds().add(113);
                indexList = klConceptFacade.index(medRetrievalVO);
                if (ListUtil.isNotEmpty(indexList)) {
                    retrievalDTO.setTransfusionNames(indexList.stream().map(i -> i.getName()).collect(Collectors.toList()));
                }
                break;
            case 9:
                medRetrievalVO.setTypeId(103);
                medRetrievalVO.getTypeIds().add(103);
                indexList = klConceptFacade.index(medRetrievalVO);
                if (ListUtil.isNotEmpty(indexList)) {
                    retrievalDTO.setSymptomNames(indexList.stream().map(i -> i.getName()).collect(Collectors.toList()));
                }
                break;
        }
        return retrievalDTO;
    }

    /**
     * 医学知识（静态信息）检索
     *
     * @param staticKnowledgeIndexVO
     * @return
     */
    public List<StaticKnowledgeIndexDTO> staticKnowledgeIndex(StaticKnowledgeIndexVO staticKnowledgeIndexVO) {
        staticKnowledgeIndexVO.setTypeIds(new ArrayList<>());
        if (staticKnowledgeIndexVO.getSize() == null) {
            staticKnowledgeIndexVO.setSize(100);
        }

        List<Integer> types = staticKnowledgeIndexVO.getTypes();

        List<DictionaryInfoDTO> dicType = dictionaryFacade.getListByGroupType(10);
        Map<String, String> dicTypeMap = EntityUtil.makeMapWithKeyValue(dicType, "name", "val");
        List<DictionaryInfoDTO> dicTypeName = dictionaryFacade.getListByGroupType(13);

        //检索类型（多选）：0-全部、1-诊断、2-药品、3-检验、4-检查、5-手术和操作
        if (ListUtil.isEmpty(types)
                || (ListUtil.isNotEmpty(types) && types.contains(0))) {
            staticKnowledgeIndexVO.getTypeIds().addAll(Arrays.asList(new Integer[] { 100, 101, 106, 107, 108, 109, 110 }));
        } else {
            if (types.contains(1)) {
                staticKnowledgeIndexVO.getTypeIds().add(100);
            }
            if (types.contains(2)) {
                staticKnowledgeIndexVO.getTypeIds().add(101);
            }
            if (types.contains(3)) {
                staticKnowledgeIndexVO.getTypeIds().add(107);
                staticKnowledgeIndexVO.getTypeIds().add(108);
            }
            if (types.contains(5)) {
                staticKnowledgeIndexVO.getTypeIds().add(109);
                staticKnowledgeIndexVO.getTypeIds().add(110);
            }
            if (types.contains(6)) {
                staticKnowledgeIndexVO.getTypeIds().add(106);
            }
        }

        List<StaticKnowledgeIndexDTO> retList = klConceptFacade.staticIndex(staticKnowledgeIndexVO);

        retList.forEach(i -> {
            i.setTypeName(convertTypeName(i.getTypeName(), 2, dicTypeName));
            if (dicTypeMap.containsKey(i.getTypeName())) {
                i.setType(Integer.valueOf(dicTypeMap.get(i.getTypeName())));
            }
        });
        return retList;
    }

    /**
     * 页面显示词性和标准词性转换
     *
     * @param typeName
     * @param flag     1-页面词性转标准词性,2- 标准词性转页面词性
     * @param dicList
     * @return
     */
    public String convertTypeName(String typeName, Integer flag, List<DictionaryInfoDTO> dicList) {
        if (ListUtil.isNotEmpty(dicList)) {
            Map<String, String> nameValMap = new HashMap<>();
            if (flag.equals(1)) {
                nameValMap = EntityUtil.makeMapWithKeyValue(dicList, "name", "val");
            } else if (flag.equals(2)) {
                nameValMap = EntityUtil.makeMapWithKeyValue(dicList, "val", "name");
            }
            if (nameValMap.containsKey(typeName)) {
                return nameValMap.get(typeName);
            }
        }
        return typeName;
    }
}