package com.diagbot.facade;

import com.diagbot.dto.PushDTO;
import com.diagbot.dto.PushPlanDTO;
import com.diagbot.dto.WordCrfDTO;
import com.diagbot.enums.DiseasePushTypeEnum;
import com.diagbot.process.PushProcess;
import com.diagbot.util.CoreUtil;
import com.diagbot.util.ParamUtil;
import com.diagbot.vo.PushPlanVO;
import com.diagbot.vo.PushVO;
import com.diagbot.vo.StandConvert;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.LinkedHashMap;
import java.util.Map;

/**
 * @Description: 推送facade
 * @author: zhoutg
 * @time: 2018/8/6 9:11
 */
@Component
public class PushFacade {

    @Autowired
    PushProcess pushProcess;
    @Autowired
    CommonFacade commonFacade;
    @Autowired
    NeoFacade neoFacade;
    @Autowired
    FollowupPlanInfoFacade followupPlanInfoFacade;
    @Autowired
    TranHospitalInfoFacade tranHospitalInfoFacade;

    public PushDTO processAggreate(PushVO pushVo) {
        // 年龄容错处理
        if (pushVo.getAgeNum() == null) {
            pushVo.setAgeNum(CoreUtil.convertAge(pushVo.getAge()));
        }
        PushDTO pushDTO = new PushDTO();
        Integer pushMode = tranHospitalInfoFacade.getPushMode(pushVo.getHospitalId());
        //全科推送，走5.0
        if (pushMode.equals(DiseasePushTypeEnum.gp.getKey())) {
            pushDTO = this.pushFacIcss(pushVo);
        } else if (pushMode.equals(DiseasePushTypeEnum.specialty.getKey())) {
            //专科推送，走6.0
            //pushDTO = this.pushFac(pushVo);
            // 2021/2/22 同老楼确认专科也走5.0
            pushDTO = this.pushFacIcss(pushVo);
        }

        return pushDTO;
    }

    /**
     * 推送调icss5.0接口
     *
     * @param pushVo
     * @return
     */
    public PushDTO pushFacIcss(PushVO pushVo) {
        Map<String, Object> debug = new LinkedHashMap<>();
        long pushStart = System.currentTimeMillis();
        PushDTO pushDTO = pushProcess.processIcss(pushVo);
        CoreUtil.getDebugStr(pushStart, "推送耗时", debug);
        pushDTO.setDebug(debug);
        return pushDTO;
    }


    /**
     * 推送业务
     *
     * @param
     * @return
     */
    public PushDTO pushFac(PushVO pushVo) {
        Map<String, Object> debug = new LinkedHashMap<>();
        WordCrfDTO wordCrfDTO = new WordCrfDTO();
        long l1 = System.currentTimeMillis();

        // 模型处理数据
        long crfStart = System.currentTimeMillis();
        wordCrfDTO = commonFacade.crf_process(pushVo);
        CoreUtil.getDebugStr(crfStart, "模型处理耗时", debug);

        // 标准词转换
        long standStart = System.currentTimeMillis();
        StandConvert standConvert = commonFacade.dataTypeGet(wordCrfDTO);
        Map<String, Map<String, String>> standConvertMap = neoFacade.standConvertCrf(standConvert);
        commonFacade.dataTypeSet(wordCrfDTO, standConvertMap);
        CoreUtil.getDebugStr(standStart, "标准词转换耗时", debug);
        ParamUtil.dealLis(wordCrfDTO.getLis());

        // 推送
        long pushStart = System.currentTimeMillis();
        PushDTO process = pushProcess.process(pushVo, standConvertMap, wordCrfDTO);
        CoreUtil.getDebugStr(pushStart, "推送耗时", debug);
        process.setDebug(debug);

        CoreUtil.getDebugStr(l1, "本次调用总计耗时", debug);
        CoreUtil.getDebugObject("数据", debug, wordCrfDTO);
        return process;
    }

    /**
     * 手术随访计划（持续检验检查）推送
     *
     * @param pushPlanVO
     * @return
     */
    public PushPlanDTO pushPlan(PushPlanVO pushPlanVO) {
        Map<String, Object> debug = new LinkedHashMap<>();
        long l1 = System.currentTimeMillis();
        PushPlanDTO pushPlanDTO = new PushPlanDTO();
        if (pushPlanVO.getOperationName() != null) {
            WordCrfDTO wordCrfDTO = new WordCrfDTO();
            wordCrfDTO.setOperationName(pushPlanVO.getOperationName());
            // 标准词转换
            long standStart = System.currentTimeMillis();
            StandConvert standConvert = commonFacade.dataTypeGet(wordCrfDTO);
            Map<String, Map<String, String>> standConvertMap = neoFacade.standConvertCrf(standConvert);
            commonFacade.dataTypeSet(wordCrfDTO, standConvertMap);
            CoreUtil.getDebugStr(standStart, "标准词转换耗时", debug);
            // 推送随访计划
            long pushStart = System.currentTimeMillis();
            pushPlanDTO = followupPlanInfoFacade.getFollowUpPlans(pushPlanVO.getOperationName(), pushPlanVO.getHospitalId());
            CoreUtil.getDebugStr(pushStart, "推送耗时", debug);
            pushPlanDTO.setDebug(debug);
        }
        CoreUtil.getDebugStr(l1, "本次调用总计耗时", debug);

        return pushPlanDTO;
    }

}
