package com.diagbot.process;

import com.diagbot.biz.push.entity.Item;
import com.diagbot.dto.BillMsg;
import com.diagbot.dto.IndicationDTO;
import com.diagbot.dto.RuleBaseDTO;
import com.diagbot.dto.RuleConditionDTO;
import com.diagbot.dto.RuleExtDTO;
import com.diagbot.dto.RuleSimpleDTO;
import com.diagbot.dto.WordCrfDTO;
import com.diagbot.enums.BaseTypeEnum;
import com.diagbot.enums.ConEnum;
import com.diagbot.enums.LexiconEnum;
import com.diagbot.enums.RuleTypeEnum;
import com.diagbot.facade.CommonFacade;
import com.diagbot.facade.NeoFacade;
import com.diagbot.model.entity.Clinical;
import com.diagbot.model.entity.Operation;
import com.diagbot.model.label.ChiefLabel;
import com.diagbot.model.label.PacsLabel;
import com.diagbot.model.label.PastLabel;
import com.diagbot.model.label.PresentLabel;
import com.diagbot.rule.AgeRule;
import com.diagbot.rule.CommonRule;
import com.diagbot.rule.DrugRule;
import com.diagbot.rule.GroupRule;
import com.diagbot.rule.LisRule;
import com.diagbot.rule.MedEquRule;
import com.diagbot.rule.PacsRule;
import com.diagbot.rule.SexRule;
import com.diagbot.rule.VitalRule;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.CoreUtil;
import com.diagbot.vo.RuleVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Set;

/**
 * @Description: 开单总入口
 * @author: zhoutg
 * @time: 2018/8/6 9:11
 */
@Component
public class BillProcess {

    @Autowired
    NeoFacade neoFacade;
    @Autowired
    DrugRule drugRule;
    @Autowired
    SexRule sexRule;
    @Autowired
    AgeRule ageRule;
    @Autowired
    CommonRule commonRule;
    @Autowired
    LisRule lisRule;
    @Autowired
    PacsRule pacsRule;
    @Autowired
    VitalRule vitalRule;
    @Autowired
    GroupRule groupRule;
    @Autowired
    MedEquRule medEquRule;
    @Autowired
    CommonFacade commonFacade;

    // 规则处理
    public void processRule(List<RuleVO> ruleVOList, WordCrfDTO wordCrfDTO, IndicationDTO indicationDTO) {
        List<BillMsg> billMsgList = indicationDTO.getBillMsgList();
        Set<String> set = new LinkedHashSet<>();    // A与B不宜同时进行, B与A不宜同时进行 只能提示一个
        for (RuleVO ruleVO : ruleVOList) {
            /** 1、获取开单项相关数据 */
            RuleExtDTO ruleExtDTO = commonFacade.getRuleData(ruleVO);
            if (ruleExtDTO == null) {
                continue;
            }

            /** 2、简化拼接对象，用于开单项提示语 */
            RuleSimpleDTO ruleSimpleDTO = new RuleSimpleDTO();
            BeanUtil.copyProperties(ruleExtDTO, ruleSimpleDTO);

            /**---------------------------3、数据统一处理开始--------------------------------- */
            ChiefLabel chiefLabel = wordCrfDTO.getChiefLabel();
            PresentLabel presentLabel = wordCrfDTO.getPresentLabel();
            PastLabel pastLabel = wordCrfDTO.getPastLabel();

            // 【过敏药品数据来源】
            List<Item> allergyMedicines = commonRule.getDrugAllergySource(wordCrfDTO);

            // 【药品数据来源】(主诉、现病史、既往史、结构化药品)
            List<Item> medicineAll = wordCrfDTO.getDrugSource();

            // 【诊断数据来源】
            List<Item> diags = wordCrfDTO.getDiagSource();

            // 【辅检结果数据来源】
            PacsLabel pacsLabel = wordCrfDTO.getPacsLabel();
            List<Item> pacsDescList = pacsLabel.getRes();

            // 【手术数据来源】
            List<Operation> operations_all = new ArrayList<>();
            List<Item> operation = wordCrfDTO.getOperation();
            List<Operation> operationsItem = new ArrayList<>();
            for (Item item : operation) {
                Operation operaData = new Operation();
                operaData.setName(item.getName());
                operaData.setStandName(item.getUniqueName());
                operationsItem.add(operaData);
            }
            CoreUtil.addList(operations_all, chiefLabel.getOperations()); // 主诉手术
            CoreUtil.addList(operations_all, presentLabel.getOperations()); // 现病史手术
            CoreUtil.addList(operations_all, pastLabel.getOperations()); // 既往史手术
            CoreUtil.addList(operations_all, operationsItem); // 结构化手术

            // 【临床表现数据来源】（主诉、现病史）
            List<Clinical> clinicals = new ArrayList<>();
            CoreUtil.addList(clinicals, chiefLabel.getClinicals()); // 主诉临床表现
            CoreUtil.addList(clinicals, presentLabel.getClinicals()); // 现病史临床表现
            /**---------------------------数据统一处理结束--------------------------------- */

            /** 4、规则处理 */
            List<RuleConditionDTO> ruleConditionDTOList = ruleExtDTO.getRuleConditionDTOList();
            for (RuleConditionDTO ruleConditionDTO : ruleConditionDTOList) {
                List<RuleBaseDTO> ruleBaseDTOList = ruleConditionDTO.getRuleBaseDTOList();
                switch (RuleTypeEnum.getEnum(ruleExtDTO.getRuleType())) {
                    case bill: // redis 数据以 _1结尾
                        for (RuleBaseDTO ruleBaseDTO : ruleBaseDTOList) {
                            if (LexiconEnum.getEnum(ruleBaseDTO.getBaseLibType()) != null) { // 避免空指针
                                switch (LexiconEnum.getEnum(ruleBaseDTO.getBaseLibType())) {
                                    case Gender: // 性别
                                        sexRule.bill(wordCrfDTO, ruleBaseDTO, billMsgList, ConEnum.gender.getName(), ruleSimpleDTO);
                                    case Age: // 年龄
                                        ageRule.bill(wordCrfDTO, ruleBaseDTO, billMsgList, ConEnum.age.getName(), ruleSimpleDTO);
                                        break;
                                    case Disease: // 诊断
                                        commonRule.compareItemWithBill(diags, ruleBaseDTO, billMsgList, ConEnum.disease.getName(), ruleSimpleDTO);
                                        break;
                                    case LisSubName: // 化验
                                        lisRule.bill(wordCrfDTO.getLis(), ruleBaseDTO, billMsgList, ConEnum.lis.getName(), ruleSimpleDTO);
                                        break;
                                    case VitalResult: //体格检查结果
                                    case Vital: // 体格检查项目
                                        vitalRule.bill(wordCrfDTO.getVitalLabel(), ruleBaseDTO, billMsgList, ConEnum.vitals.getName(), ruleSimpleDTO);
                                        break;
                                    case PacsName: // 辅检项目
                                        if (ruleBaseDTO.getBaseType() != null && ruleBaseDTO.getBaseType().equals(BaseTypeEnum.B5.getKey())) {
                                            // 开单项互斥
                                            commonRule.exclusionBill(wordCrfDTO, ruleBaseDTO, billMsgList, ConEnum.exclusion.getName(), ruleSimpleDTO, set);
                                        } else {
                                            // 辅检项目,暂无数据
                                            pacsRule.bill(wordCrfDTO.getPacs(), ruleBaseDTO, billMsgList, ConEnum.pacs.getName(), ruleSimpleDTO);
                                        }
                                        break;
                                    case Symptom: // 临床表现
                                        commonRule.compareNameWithBill(clinicals, ruleBaseDTO, billMsgList, ConEnum.clinicfindings.getName(), ruleSimpleDTO);
                                        break;
                                    case Operation: // 手术(既往史、现病史、医嘱手术)
                                        commonRule.compareNameWithBill(operations_all, ruleBaseDTO, billMsgList, ConEnum.operations.getName(), ruleSimpleDTO);
                                        break;
                                    case Medicine:
                                    case MedChemClass:
                                    case MedZhiLiaoClass:
                                    case MedYaoLiClass:
                                    case MedJiePouClass:
                                        if (ruleBaseDTO.getBaseType().equals(4)) {
                                            // 禁忌过敏药品（既往史）
                                            drugRule.bill(allergyMedicines, ruleBaseDTO, billMsgList, ConEnum.allergicmeds.getName(), ruleSimpleDTO);
                                        } else {
                                            // 服用药品（主诉、现病史、既往史、结构化药品）
                                            drugRule.bill(medicineAll, ruleBaseDTO, billMsgList, ConEnum.oralmeds.getName(), ruleSimpleDTO);
                                        }
                                        break;
                                    case Allergen: // 其他过敏原
                                        drugRule.bill(allergyMedicines, ruleBaseDTO, billMsgList, ConEnum.otherAllergy.getName(), ruleSimpleDTO);
                                        break;
                                    case Group: // 禁忌人群
                                        groupRule.bill(wordCrfDTO, ruleBaseDTO, billMsgList, ConEnum.group.getName(), ruleSimpleDTO);
                                        break;
                                    case PacsResult: // 禁用辅助检查描述
                                        commonRule.compareItemWithBill(pacsDescList, ruleBaseDTO, billMsgList, ConEnum.pacsDesc.getName(), ruleSimpleDTO);
                                        break;
                                    case Device: // 禁忌医疗器械及物品
                                        medEquRule.bill(wordCrfDTO, ruleBaseDTO, billMsgList, ConEnum.medEqu.getName(), ruleSimpleDTO);
                                        break;
                                    default:
                                        break;
                                }
                            }
                        }
                        break;
                    case repeat:  // redis 数据以 _6结尾
                        for (RuleBaseDTO ruleBaseDTO : ruleBaseDTOList) {
                            // 无需重复开单项
                            commonRule.needlessRepeatOrder(wordCrfDTO, ruleBaseDTO, billMsgList, ConEnum.repeat.getName(), ruleSimpleDTO);
                        }
                        break;
                    default:
                        break;
                }
            }
        }
        // 24小时重复开单项
        commonRule.repeat24Bill(wordCrfDTO, billMsgList);
    }
}
