package com.diagbot.process;

import com.diagbot.biz.push.entity.Item;
import com.diagbot.biz.push.entity.Lis;
import com.diagbot.biz.push.entity.Pacs;
import com.diagbot.client.IcssPushServiceClient;
import com.diagbot.dto.NeoPushDTO;
import com.diagbot.dto.PushBaseDTO;
import com.diagbot.dto.PushDTO;
import com.diagbot.dto.TreatDTO;
import com.diagbot.dto.WordCrfDTO;
import com.diagbot.dto.icss.FeatureRate;
import com.diagbot.dto.icss.Response;
import com.diagbot.dto.icss.ResponseData;
import com.diagbot.entity.PreResult;
import com.diagbot.enums.DiseaseTypeEnum;
import com.diagbot.enums.MedicalAdviceEnum;
import com.diagbot.enums.PushRelationTypeEnum;
import com.diagbot.enums.RedisEnum;
import com.diagbot.enums.StandConvertEnum;
import com.diagbot.facade.CommonFacade;
import com.diagbot.facade.KlRelationFacade;
import com.diagbot.facade.NeoFacade;
import com.diagbot.facade.TranLisConfigIcssFacade;
import com.diagbot.util.ListUtil;
import com.diagbot.util.RedisUtil;
import com.diagbot.vo.DiseaseItemVO;
import com.diagbot.vo.IcssVo;
import com.diagbot.vo.NeoPushVO;
import com.diagbot.vo.PushVO;
import com.diagbot.vo.neoPushEntity.Diag;
import com.diagbot.vo.neoPushEntity.DiagVo;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author kwz
 * @date 2020/8/17
 * @time 19:42
 */
@Component
public class PushProcess {

    @Autowired
    NeoFacade neoFacade;
    @Autowired
    CommonFacade commonFacade;
    @Autowired
    TranLisConfigIcssFacade tranLisConfigIcssFacade;
    @Autowired
    IcssPushServiceClient icssPushServiceClient;
    @Autowired
    RedisUtil redisUtil;
    @Autowired
    private KlRelationFacade klRelationFacade;

    public PushDTO processIcss(PushVO pushVo) {

        int length = pushVo.getLength();
        Map<String, List<String>> typeWords = Maps.newHashMap();
        List<String> ruleTypeList = Arrays.asList(pushVo.getFeatureType().split(","));
        List<String> pushDiags = Lists.newArrayList();
        Map<String, Integer> diag_sign = new LinkedHashMap<>();
        PushDTO pushDTO = new PushDTO();
        IcssVo icssVo = pushVo2IcssVo(pushVo);
//        String s = JSONObject.toJSONString(icssVo);
        if ((StringUtils.isNotBlank(icssVo.getChief()) && icssVo.getChief().trim().equals("呼吸困难"))
                || (StringUtils.isNotBlank(icssVo.getSymptom()) && icssVo.getSymptom().trim().equals("呼吸困难"))) {
            icssVo.setChief("呼吸困难。");
        }
        Response<ResponseData> annotation = icssPushServiceClient.getAnnotation(icssVo);
        if (annotation != null) {
            ResponseData data = annotation.getData();
            if (data != null) {
                Map<String, Map<String, String>> inputs = data.getInputs();
                if (MapUtils.isNotEmpty(inputs)) {
                    List<String> words = new ArrayList<>(inputs.keySet());
                    typeWords.put(StandConvertEnum.symptom.toString(), words);
                    typeWords.put(StandConvertEnum.disease.toString(), words);
                    typeWords.put(StandConvertEnum.pacs.toString(), words);
                    typeWords.put(StandConvertEnum.lis.toString(), words);
                    typeWords.put(StandConvertEnum.vital.toString(), words);
                }
                List<FeatureRate> dis = data.getDis();
                if (ListUtil.isNotEmpty(dis)) {
                    pushDiags = dis.stream().filter(x -> x.getDesc() == null || x.getDesc().contains("确诊") || x.getDesc().contains("拟诊"))
                            .map(x -> {
                                        String diag = x.getFeatureName();
                                        Integer sign = 0;
                                        Map<String, Object> diag_signs = (Map<String, Object>) redisUtil.get(RedisEnum.diseaseCorrespond.getName() + x.getFeatureName());
                                        if (MapUtils.isNotEmpty(diag_signs)) {
                                            diag = (String) diag_signs.get("diag_icd");
                                            sign = (Integer) diag_signs.get("sign");
                                        }
                                        diag_sign.put(diag, sign);
                                        return diag;
                                    }
                            )
                            .collect(Collectors.toList());
                }
            }
        }
        Item diaeaseName = pushVo.getDiseaseName();
        Map<String, List<PushBaseDTO>> dis = new HashMap<>();
        if (ruleTypeList.contains("10") && diaeaseName != null) {
            List<TreatDTO> collect = new ArrayList<>();
            TreatDTO treatDTO = new TreatDTO();
            treatDTO.setName(diaeaseName.getName());
            String uniqueName = diaeaseName.getUniqueName();
            uniqueName = StringUtils.isNotBlank(uniqueName) ? uniqueName : diaeaseName.getName();
            treatDTO.setUniqueName(uniqueName);
            collect.add(treatDTO);
            pushDTO.setTreat(collect);
        }
        Map<String, List<String>> reverseInfoByDisName = null;
        //推送治疗
        if (diaeaseName != null) {
            String uniqueName = diaeaseName.getUniqueName();
            uniqueName = StringUtils.isNotBlank(uniqueName) ? uniqueName : diaeaseName.getName();
            //reverseInfoByDisName = neoFacade.reverseInfoByDisName(uniqueName);
            //查询疾病相关的项目
            DiseaseItemVO diseaseItemVO = new DiseaseItemVO();
            diseaseItemVO.setDiseaseName(uniqueName);
            Map<Long, List<String>> items = klRelationFacade.getItemByDiseaseMap(diseaseItemVO);
            if (MapUtils.isNotEmpty(items)){
                if (ruleTypeList.contains("8") && ListUtil.isNotEmpty(items.get(PushRelationTypeEnum.DRUG.getKey()))){
                    pushDTO.setMedicines(items.get(PushRelationTypeEnum.DRUG.getKey()).stream().map(x -> {
                        PushBaseDTO pushBaseDTO = new PushBaseDTO();
                        pushBaseDTO.setName(x);
                        return pushBaseDTO;
                    }).collect(Collectors.toList()));
                }
                if (ruleTypeList.contains("9") && ListUtil.isNotEmpty(items.get(PushRelationTypeEnum.OPERATION.getKey()))){
                    pushDTO.setOperations(items.get(PushRelationTypeEnum.OPERATION.getKey()).stream().map(x -> {
                        PushBaseDTO pushBaseDTO = new PushBaseDTO();
                        pushBaseDTO.setName(x);
                        return pushBaseDTO;
                    }).collect(Collectors.toList()));
                }
            }
        }
//        if (reverseInfoByDisName != null) {
//            if (ruleTypeList.contains("8") && reverseInfoByDisName.containsKey(StaticTypeEnum.DRUG.getName())) {
//                pushDTO.setMedicines(reverseInfoByDisName.get(StaticTypeEnum.DRUG.getName()).stream().map(x -> {
//                    PushBaseDTO pushBaseDTO = new PushBaseDTO();
//                    pushBaseDTO.setName(x);
//                    return pushBaseDTO;
//                }).collect(Collectors.toList()));
//            }
//            if (ruleTypeList.contains("9") && reverseInfoByDisName.containsKey(StaticTypeEnum.OPT.getName())) {
//                pushDTO.setOperations(reverseInfoByDisName.get(StaticTypeEnum.OPT.getName()).stream().map(x -> {
//                    PushBaseDTO pushBaseDTO = new PushBaseDTO();
//                    pushBaseDTO.setName(x);
//                    return pushBaseDTO;
//                }).collect(Collectors.toList()));
//            }
//        }
        if (ListUtil.isNotEmpty(icssVo.getDiagOrder())) {
            List<PreResult> diagOrder = icssVo.getDiagOrder();
            //把第一个推送出来的诊断set到diagOrder中，再反推
            NeoPushVO pushVO = new NeoPushVO();
            DiagVo diagVo = new DiagVo();
            diagVo.setDiags(diagOrder.stream().map(x -> {
                String name = x.getName();
                Diag diag = new Diag();
                Map<String, Object> diag_signs = (Map<String, Object>) redisUtil.get(RedisEnum.diseaseCorrespond.getName() + name);
                name = MapUtils.isNotEmpty(diag_signs)? (String) diag_signs.get("diag_icd"):name;
                diag.setName(name);
                return diag;
            }).collect(Collectors.toList()));
            pushVO.setDiagVo(diagVo);
            reversePushPackage(length, pushDTO, ruleTypeList, typeWords, pushVO, dis);
        }
        try {
            boolean allFieldNull = isAllFieldNull(pushDTO);
            if (allFieldNull) {
                if (ListUtil.isNotEmpty(pushDiags)) {
                    //先把所有疾病推出来
                    if (ruleTypeList.contains("7")) {
                        if(MapUtils.isNotEmpty(diag_sign)){
                            Map.Entry<String, Integer> first = diag_sign.entrySet().stream().findFirst().get();
                            if (first.getValue() == 1){
                                List<String> firstdiag = Lists.newArrayList(first.getKey());
                                List<PushBaseDTO> collect_highrisk = firstdiag.stream().map(x -> {
                                    PushBaseDTO pushBaseDTO = new PushBaseDTO();
                                    pushBaseDTO.setName(x);
                                    return pushBaseDTO;
                                }).collect(Collectors.toList());
                                dis.put(DiseaseTypeEnum.highriskDis.getName(), collect_highrisk);
                            }

                        }

                        List<PushBaseDTO> diseeases = pushDiags.stream().map(x -> {
                            PushBaseDTO pushBaseDTO = new PushBaseDTO();
                            pushBaseDTO.setName(x);
                            return pushBaseDTO;
                        }).collect(Collectors.toList());
                        diseeases = diseeases.subList(0, diseeases.size() >= length ? length : diseeases.size());
                        dis.put(DiseaseTypeEnum.possibleDis.getName(), diseeases);
                        pushDTO.setDis(dis);

                    }
                    //把第一个推送出来的诊断set到diagOrder中，再反推
                    NeoPushVO pushVO = new NeoPushVO();
                    DiagVo diagVo = new DiagVo();
                    diagVo.setDiags(pushDiags.stream().map(x -> {
                        Diag diag = new Diag();
                        diag.setName(x);
                        return diag;
                    }).collect(Collectors.toList()));
                    pushVO.setDiagVo(diagVo);
                    //调用反推
                    reversePushPackage(length, pushDTO, ruleTypeList, typeWords, pushVO, dis);

                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }


        return pushDTO;
    }

    //判断该对象是否: 返回ture表示所有属性为null  返回false表示不是所有属性都是null
    public static boolean isAllFieldNull(Object obj) throws Exception {
        List<String> params = Lists.newArrayList("operations", "medicines", "treat");
        Class stuCla = (Class) obj.getClass();// 得到类对象
        Field[] fs = stuCla.getDeclaredFields();//得到属性集合
        boolean flag = true;
        for (Field f : fs) {//遍历属性
            f.setAccessible(true); // 设置属性是可以访问的(私有的也可以)
            String name = f.getName();
            Object val = f.get(obj);// 得到此属性的值
            if (val != null && !params.contains(name)) {//只要有1个属性不为空,那么就不是所有的属性值都为空
                flag = false;
                break;
            }
        }
        return flag;
    }

    /**
     * pushVo转Icss入参
     *
     * @param pushVo
     * @return
     */
    public IcssVo pushVo2IcssVo(PushVO pushVo) {
        IcssVo icssVo = new IcssVo();
        icssVo.setAge(new Double(pushVo.getAgeNum()).intValue());
        String sexs = "";
        Integer sex = pushVo.getSex();
        if (sex == 1) {
            sexs = "M";
        } else if (sex == 2) {
            sexs = "F";
        }
        icssVo.setSex(sexs);
        icssVo.setChief(pushVo.getChief());
        icssVo.setSymptom(pushVo.getSymptom());
        icssVo.setVital(pushVo.getVital());
        icssVo.setPasts(pushVo.getPasts());
        icssVo.setPersonal(pushVo.getPersonal());
        icssVo.setFeatureType(pushVo.getFeatureType() + ",7");

        List<Lis> lis = pushVo.getLis();
        lis = tranLisConfigIcssFacade.addLisUniqueName(lis, "A001");
        List<PreResult> lis_icss = lis.stream().map(x -> {
            PreResult instantiate = BeanUtils.instantiate(PreResult.class);
            BeanUtils.copyProperties(x, instantiate);
            instantiate.setValue(x.getValue() == null ? "" : x.getValue().toString());
            instantiate.setMaxValue(x.getMaxValue() == null ? "" : x.getMaxValue().toString());
            instantiate.setMinValue(x.getMinValue() == null ? "" : x.getMinValue().toString());
            return instantiate;
        }).collect(Collectors.toList());
        icssVo.setLis(lis_icss);

        List<Pacs> pacs = pushVo.getPacs();
        if (ListUtil.isNotEmpty(pacs)) {
            List<PreResult> pacs_icss = pacs.stream().map(x -> {
                PreResult instantiate = new PreResult();
                BeanUtils.copyProperties(x, instantiate);
                return instantiate;
            }).collect(Collectors.toList());
            icssVo.setPacs(pacs_icss);
        }

        if (ListUtil.isNotEmpty(pushVo.getDiag())) {
            List<PreResult> diagOrder = pushVo.getDiag().stream().map(x -> {
                String name = x.getName();
                PreResult preResult = new PreResult();
                preResult.setName(name);
                preResult.setValue(name);
                return preResult;
            }).collect(Collectors.toList());
            icssVo.setDiagOrder(diagOrder);
        }
        return icssVo;
    }

    public PushDTO process(PushVO pushVo, Map<String, Map<String, String>> standConvertMap, WordCrfDTO wordCrfDTO) {
        int length = pushVo.getLength();
        PushDTO pushDTO = new PushDTO();
//        List<String> ruleTypeList = Arrays.asList(pushVo.getFeatureType().split(","));
//        Map<String, List<String>> typeWords = typeWords(standConvertMap);
//        //生成push入参，供图谱调用
//        NeoPushVO pushVO = commonFacade.generatePushInput(wordCrfDTO);
//        Map<String, List<PushBaseDTO>> dis = new HashMap<>();
//        List<String> push = neoFacade.getPush(pushVO);
//        if (ListUtil.isNotEmpty(push)) {
//            //先把所有疾病推出来
//            if (ruleTypeList.contains("7")) {
//                List<PushBaseDTO> diseeases = push.stream().map(x -> {
//                    PushBaseDTO pushBaseDTO = new PushBaseDTO();
//                    pushBaseDTO.setName(x);
//                    return pushBaseDTO;
//                }).collect(Collectors.toList());
//                diseeases = diseeases.subList(0, diseeases.size() >= length ? length : diseeases.size());
//                dis.put(DiseaseTypeEnum.possibleDis.getName(), diseeases);
//                pushDTO.setDis(dis);
//            }
//            //把第一个推送出来的诊断set到diagOrder中，再反推
//            setPushVo(pushVO, push);
//            //调用反推
//            reversePushPackage(length, pushDTO, ruleTypeList, typeWords, pushVO, dis);
//        }
        return pushDTO;

    }

    private void setPushVo(NeoPushVO pushVO, List<String> push) {
        String firstDisease = push.get(0);
        List<Diag> diags = new ArrayList<>();
        Diag diag = new Diag();
        diag.setName(firstDisease);
        diags.add(diag);
        DiagVo diagVo = new DiagVo();
        diagVo.setDiags(diags);
        pushVO.setDiagVo(diagVo);
    }

    public Map<String, List<String>> typeWords(Map<String, Map<String, String>> standConvertMap) {
        Map<String, List<String>> typeWordsMap = new HashMap<>();
        for (Map.Entry<String, Map<String, String>> s : standConvertMap.entrySet()) {
            List<String> words = new ArrayList<>();
            String type = s.getKey();
            Map<String, String> value = s.getValue();
            value.forEach((name, standName) -> {
                words.add(name);
                words.add(standName);
            });
            typeWordsMap.put(type, new ArrayList<>(new HashSet<>(words)));
        }

        return typeWordsMap;
    }

    private void reversePushPackage(){

    }

    private void reversePushPackage(int length, PushDTO pushDTO, List<String> ruleTypeList, Map<String, List<String>> typeWords, NeoPushVO pushVO, Map<String, List<PushBaseDTO>> dis) {
        NeoPushDTO reversePush = neoFacade.getReversePush(pushVO);
        // 症状
        if (ruleTypeList.contains("1")) {
            if (ListUtil.isNotEmpty(reversePush.getSymptoms())) {
                List<String> symptoms = reversePush.getSymptoms().stream().map(x -> x.getName()).collect(Collectors.toList());
                if (symptoms.size() > 0) {
                    List<PushBaseDTO> filtervitals = getPackagePushBaseDTO(typeWords, symptoms, StandConvertEnum.symptom);
                    pushDTO.setSymptom(filtervitals.subList(0, filtervitals.size() >= length ? length : filtervitals.size()));
                }
            }
        }
        // 查体
        if (ruleTypeList.contains("4")) {
            if (ListUtil.isNotEmpty(reversePush.getVitals())) {
                List<String> vitals = reversePush.getVitals().stream().map(x -> x.getName()).collect(Collectors.toList());
                if (vitals.size() > 0) {
                    List<PushBaseDTO> filtervitals = getPackagePushBaseDTO(typeWords, vitals, StandConvertEnum.vital);
                    pushDTO.setVital(filtervitals.subList(0, filtervitals.size() >= length ? length : filtervitals.size()));
                }
            }
        }
        // 化验
        if (ruleTypeList.contains("5")) {
            if (ListUtil.isNotEmpty(reversePush.getLis())) {
                List<String> lises = reversePush.getLis().stream().map(x -> x.getName()).collect(Collectors.toList());
                if (lises.size() > 0) {
                    List<PushBaseDTO> filterlis = getPackagePushBaseDTO(typeWords, lises, StandConvertEnum.lis);
                    pushDTO.setLis(filterlis.subList(0, filterlis.size() >= length ? length : filterlis.size()));
                }
            }
        }
        // 辅检
        if (ruleTypeList.contains("6")) {
            if (ListUtil.isNotEmpty(reversePush.getPacs())) {
                List<String> pacses = reversePush.getPacs().stream().map(x -> x.getName()).collect(Collectors.toList());
                if (pacses.size() > 0) {
                    List<PushBaseDTO> filterpacs = getPackagePushBaseDTO(typeWords, pacses, StandConvertEnum.pacs);
                    pushDTO.setPacs(filterpacs.subList(0, filterpacs.size() >= length ? length : filterpacs.size()));
                }
            }
        }
        // 药品
        if (ruleTypeList.contains("8")) {
            if (reversePush.getTreat() != null && ListUtil.isNotEmpty(reversePush.getTreat().get(MedicalAdviceEnum.drug.getName()))) {
                List<String> drugs = reversePush.getTreat().get(MedicalAdviceEnum.drug.getName()).stream().map(x -> x.getName()).collect(Collectors.toList());
                /*reversePush.stream().filter(x ->x.getTreat() != null && ListUtil.isNotEmpty(x.getTreat().get(MedicalAdviceEnum.drug.getName())))
                        .forEach(x -> {
                            List<String> drugList = x.getTreat().get(MedicalAdviceEnum.drug.getName()).stream().map(y -> y.getName()).collect(Collectors.toList());
                            drugs.removeAll(drugList);
                            drugs.addAll(drugList);
                        });*/
                if (drugs.size() > 0) {
                    List<PushBaseDTO> filterdrugs = getPackagePushBaseDTO(typeWords, drugs, StandConvertEnum.drug);
                    pushDTO.setMedicines(filterdrugs.subList(0, filterdrugs.size() >= length ? length : filterdrugs.size()));
                }
            }
        }
        // 手术
        if (ruleTypeList.contains("9")) {
            if (reversePush.getTreat() != null && ListUtil.isNotEmpty(reversePush.getTreat().get(MedicalAdviceEnum.operation.getName()))) {

                List<String> operations = reversePush.getTreat().get(MedicalAdviceEnum.operation.getName()).stream().map(x -> x.getName()).collect(Collectors.toList());
                if (operations.size() > 0) {
                    List<PushBaseDTO> filteroperations = getPackagePushBaseDTO(typeWords, operations, StandConvertEnum.operation);
                    pushDTO.setOperations(filteroperations.subList(0, filteroperations.size() >= length ? length : filteroperations.size()));
                }
            }
        }
        //反推，推鉴别诊断
        if (ruleTypeList.contains("7")) {
            if (ListUtil.isNotEmpty(reversePush.getDifferentialDis())) {
                List<String> differentDis = reversePush.getDifferentialDis().stream().map(x -> x.getName()).collect(Collectors.toList());
                if (differentDis.size() > 0) {
                    List<PushBaseDTO> filterpacs = getPackagePushBaseDTO(typeWords, differentDis, StandConvertEnum.disease);
                    List<PushBaseDTO> pushDiffBaseDTOS = filterpacs.subList(0, filterpacs.size() >= length ? length : filterpacs.size());
                    dis.put(DiseaseTypeEnum.identify.getName(), pushDiffBaseDTOS);
                    pushDTO.setDis(dis);
                }
            }
        }

    }

    private List<PushBaseDTO> getPackagePushBaseDTO(Map<String, List<String>> typeWords, List<String> symptoms, StandConvertEnum symptom) {
        List<PushBaseDTO> symptomPush = symptoms.stream()
                .map(x -> {
                    x = ListUtil.isNotEmpty(typeWords.get(symptom.toString())) ? !typeWords.get(symptom.toString()).contains(x) ? x : "" : x;
                    return x;
                }).filter(x -> StringUtils.isNotBlank(x)).map(x -> {
                    PushBaseDTO pushBaseDTO = new PushBaseDTO();
                    pushBaseDTO.setName(x);
                    return pushBaseDTO;
                }).collect(Collectors.toList());
        return symptomPush;
    }
}

