package com.diagbot.rule;

import com.diagbot.dto.BillMsg;
import com.diagbot.dto.RuleBaseDTO;
import com.diagbot.dto.RuleSimpleDTO;
import com.diagbot.dto.WordCrfDTO;
import com.diagbot.util.CoreUtil;
import com.diagbot.util.MsgUtil;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * @description: 年龄规则
 * @author: zhoutg
 * @time: 2020/8/3 14:47
 */
@Component
public class AgeRule {

    /**
     * 比较年龄——开单项
     *
     * @param wordCrfDTO
     * @param ruleBaseDTO
     * @param billMsgList
     * @param conType
     * @param ruleSimpleDTO
     */
    public void bill(WordCrfDTO wordCrfDTO, RuleBaseDTO ruleBaseDTO, List<BillMsg> billMsgList, String conType, RuleSimpleDTO ruleSimpleDTO) {
        Boolean flag = getFlag(wordCrfDTO, ruleBaseDTO);
        if (flag) {
            String message = "年龄" + wordCrfDTO.getAge();
            BillMsg billMsg = MsgUtil.getCommonBillMsg(ruleSimpleDTO.getInputName(), ruleSimpleDTO.getLibName(),
                    message, conType, ruleSimpleDTO.getLibTypeName());
            billMsgList.add(billMsg);
        }
    }

    /**
     * 比较年龄——高危手术
     *
     * @param wordCrfDTO
     * @param ruleBaseDTO
     * @param highRiskList
     * @param ruleSimpleDTO
     */
    public void highOperation(WordCrfDTO wordCrfDTO, RuleBaseDTO ruleBaseDTO, List<BillMsg> highRiskList, RuleSimpleDTO ruleSimpleDTO) {
        Boolean flag = getFlag(wordCrfDTO, ruleBaseDTO);
        if (flag) {
            String message = "年龄:" + wordCrfDTO.getAge();
            BillMsg billMsg = MsgUtil.getCommonHighRiskMsg(message, ruleSimpleDTO.getInputName(), ruleSimpleDTO.getLibTypeName());
            highRiskList.add(billMsg);
        }
    }

    /**
     * 匹配规则
     *
     * @param wordCrfDTO
     * @param ruleBaseDTO
     * @return
     */
    public boolean getFlag(WordCrfDTO wordCrfDTO, RuleBaseDTO ruleBaseDTO) {
        Boolean flag = false;
        if (wordCrfDTO.getAgeNum() != null) {
            Double age = wordCrfDTO.getAgeNum();
            flag = CoreUtil.compareNum(ruleBaseDTO, age);
        }
        return flag;
    }

}
