package com.diagbot.rule;

import com.diagbot.biz.push.entity.Item;
import com.diagbot.biz.push.entity.Pacs;
import com.diagbot.dto.BillMsg;
import com.diagbot.dto.HighRiskNeoDTO;
import com.diagbot.dto.NodeNeoDTO;
import com.diagbot.dto.RuleBaseDTO;
import com.diagbot.dto.RuleSimpleDTO;
import com.diagbot.dto.WordCrfDTO;
import com.diagbot.enums.ConEnum;
import com.diagbot.enums.TypeEnum;
import com.diagbot.model.entity.Negative;
import com.diagbot.util.CatalogueUtil;
import com.diagbot.util.CoreUtil;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.MsgUtil;
import com.diagbot.util.StringUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * @description: 通用规则提取
 * @author: kwz
 * @time: 2020/8/3 14:47
 */
@Component
public class CommonRule {

    /**
     * 比较阳性属性是否匹配
     *
     * @param input
     * @param ruleBaseDTO
     * @param billMsgList
     * @param conType
     * @param ruleSimpleDTO
     * @param <T>
     */
    public <T> void compareNameWithBill(List<T> input, RuleBaseDTO ruleBaseDTO, List<BillMsg> billMsgList,
                                        String conType, RuleSimpleDTO ruleSimpleDTO) {
        if (ListUtil.isNotEmpty(input)) {
            for (T d : input) {
                Negative val = (Negative) CoreUtil.getFieldValue(d, "negative");
                if (val == null) {
                    String c = (String) CoreUtil.getFieldValue(d, "standName");
                    String c_name = (String) CoreUtil.getFieldValue(d, "name");
                    if (StringUtils.isNotBlank(c) && CoreUtil.compareName(ruleBaseDTO, c)) {
                        BillMsg commonBillMsg = MsgUtil.getCommonBillMsg(
                                ruleSimpleDTO.getInputName(), ruleSimpleDTO.getLibName(),
                                c_name, conType, ruleSimpleDTO.getLibTypeName());
                        billMsgList.add(commonBillMsg);
                    }
                }
            }
        }
    }

    /**
     * 比较是否包含
     *
     * @param input
     * @param ruleBaseDTO
     * @return
     */
    public <T> Boolean containsItem(List<T> input, RuleBaseDTO ruleBaseDTO) {
        if (ListUtil.isNotEmpty(input) && ruleBaseDTO != null) {
            for (T t : input) {
                String c = (String) CoreUtil.getFieldValue(t, "uniqueName"); // 标准名称
                if (c.contains(ruleBaseDTO.getBaseLibName())) {
                    return true;
                }
            }
        }
        return false;
    }

    /**
     * 比较名称是否匹配
     *
     * @param input
     * @param ruleBaseDTO
     * @param <T>
     */
    public <T> Map compareItem(List<T> input, RuleBaseDTO ruleBaseDTO) {
        Map<String, Object> map = new LinkedHashMap<>();
        boolean flag = false;
        if (ruleBaseDTO == null || ListUtil.isEmpty(input)) {
            map.put("flag", flag);
            return map;
        }
        List<String> msgList = new ArrayList<>();
        for (T t : input) {
            String c = (String) CoreUtil.getFieldValue(t, "uniqueName"); // 标准名称
            String c_name = (String) CoreUtil.getFieldValue(t, "name"); // 界面名称
            if (CoreUtil.compareName(ruleBaseDTO, c)) {
                msgList.add(c_name);
                flag = true;
            }
        }
        map.put("flag", flag);
        map.put("msgList", msgList);
        return map;
    }

    /**
     * 比较名称是否匹配
     *
     * @param input
     * @param ruleBaseDTO
     * @param billMsgList
     * @param conType
     * @param ruleSimpleDTO
     * @param <T>
     */
    public <T> void compareItemWithBill(List<T> input, RuleBaseDTO ruleBaseDTO, List<BillMsg> billMsgList,
                                        String conType, RuleSimpleDTO ruleSimpleDTO) {
        if (ListUtil.isNotEmpty(input)) {
            for (T t : input) {
                String c = (String) CoreUtil.getFieldValue(t, "uniqueName"); // 标准名称
                String c_name = (String) CoreUtil.getFieldValue(t, "name"); // 界面名称
                if (CoreUtil.compareName(ruleBaseDTO, c)) {
                    BillMsg commonBillMsg = MsgUtil.getCommonBillMsg(
                            ruleSimpleDTO.getInputName(), ruleSimpleDTO.getLibName(),
                            c_name, conType, ruleSimpleDTO.getLibTypeName());
                    billMsgList.add(commonBillMsg);
                }
            }
        }
    }

    /**
     * 比较名称是否匹配
     *
     * @param input
     * @param ruleBaseDTO
     * @param highRiskList
     * @param ruleSimpleDTO
     * @param <T>
     */
    public <T> void compareItemWithHighRisk(List<T> input, RuleBaseDTO ruleBaseDTO, List<BillMsg> highRiskList, RuleSimpleDTO ruleSimpleDTO) {
        if (ListUtil.isNotEmpty(input)) {
            for (T t : input) {
                String c = (String) CoreUtil.getFieldValue(t, "uniqueName"); // 标准名称
                String c_name = (String) CoreUtil.getFieldValue(t, "name"); // 界面名称
                if (CoreUtil.compareName(ruleBaseDTO, c)) {
                    BillMsg billMsg = MsgUtil.getCommonHighRiskMsg(c_name, ruleSimpleDTO.getInputName(), ruleSimpleDTO.getLibTypeName());
                    highRiskList.add(billMsg);
                }
            }
        }
    }

    /**
     * 无需重复开单项
     *
     * @param wordCrfDTO
     * @param ruleBaseDTO
     * @param billMsgList
     * @param ruleSimpleDTO
     */
    public void needlessRepeatOrder(WordCrfDTO wordCrfDTO, RuleBaseDTO ruleBaseDTO, List<BillMsg> billMsgList, String conType, RuleSimpleDTO ruleSimpleDTO) {
        String orderStandName = ruleBaseDTO.getBaseLibName();
        Map<String, List<Pacs>> map = EntityUtil.makeEntityListMap(wordCrfDTO.getPacs(), "uniqueName");
        List<Pacs> pacsList = map.get(orderStandName);
        if (ListUtil.isNotEmpty(pacsList)) {
            sortByProperty(pacsList, "dateValue");
            Pacs pacs = pacsList.get(pacsList.size() - 1); // 按时间排序，取最后一条
            String result = pacs.getResult();
            if (StringUtil.isNotBlank(result) && StringUtil.isNotBlank(ruleBaseDTO.getBaseEqValue())
                    && getRegexRes(result, ruleBaseDTO.getBaseEqValue())) {
                String dateValue = pacs.getDateValue(); // 结果日期
                String orderDateValue = ruleSimpleDTO.getDateValue(); // 开单项日期
                if (StringUtil.isNotBlank(dateValue) && StringUtil.isNotBlank(orderDateValue)) {
                    Date dateValueDate = CatalogueUtil.parseStringDate(dateValue);
                    Date orderDateValueDate = CatalogueUtil.parseStringDate(orderDateValue);
                    if (dateValueDate != null && orderDateValueDate != null) {
                        if (!CatalogueUtil.compareTime(dateValueDate, orderDateValueDate, 60L * 24 * 7, false)) {
                            BillMsg commonBillMsg = MsgUtil.getCommonBillMsg(
                                    ruleSimpleDTO.getInputName(), ruleSimpleDTO.getLibName(),
                                    pacs.getName(), conType, ruleSimpleDTO.getLibTypeName());
                            billMsgList.add(commonBillMsg);
                        }
                    }
                }
            }
        }
    }

    /**
     * 是否有符合的数据(公共方法)
     *
     * @param content 文本内容
     * @param regex   表达式
     * @return
     */
    public Boolean getRegexRes(String content, String regex) {
        // 是否有符合的数据
        try {
            if (StringUtil.isBlank(content) || StringUtil.isBlank(regex)) {
                return false;
            }
            Pattern pattern = Pattern.compile(regex, Pattern.CASE_INSENSITIVE);
            Matcher matcher = pattern.matcher(content);
            if (matcher.find()) {
                return true;
            }
        } catch (Exception e) {
            return false;
        }
        return false;
    }

    /**
     * 根据指定字段按照字符串排序
     *
     * @param tList
     * @param property
     * @param <T>
     */
    public <T> void sortByProperty(List<T> tList, String property) {
        if (ListUtil.isNotEmpty(tList) && tList.size() > 1) {
            // items 按照时间排序
            Collections.sort(tList, new Comparator<T>() {
                @Override
                public int compare(T o1, T o2) {
                    String o1Str = (String) CoreUtil.getFieldValue(o1, property);
                    String o2Str = (String) CoreUtil.getFieldValue(o2, property);
                    if (StringUtil.isBlank(o1Str)) {
                        return -1;
                    }
                    if (StringUtil.isBlank(o2Str)) {
                        return 1;
                    }
                    return o1Str.compareTo(o2Str);
                }
            });
        }
    }

    /**
     * 24小时重复开单总入口
     *
     * @param wordCrfDTO
     * @param billMsgList
     */
    public void repeat24Bill(WordCrfDTO wordCrfDTO, List<BillMsg> billMsgList) {
        repeat24BillWithType(billMsgList, wordCrfDTO.getLisOrder(), TypeEnum.lis.getName(), ConEnum.repeat24.getName()); // 化验重复开单
        repeat24BillWithType(billMsgList, wordCrfDTO.getPacsOrder(), TypeEnum.pacs.getName(), ConEnum.repeat24.getName()); // 辅检重复开单
        // repeat24BillWithType(billMsgList, wordCrfDTO.getDrugOrder(), TypeEnum.drug.getName(), ConEnum.repeat24.getName()); // 药品重复开单
        // repeat24BillWithType(billMsgList, wordCrfDTO.getOperationOrder(), TypeEnum.operation.getName(), ConEnum.repeat24.getName()); // 手术重复开单
        // repeat24BillWithType(billMsgList, wordCrfDTO.getTransfusionOrder(), TypeEnum.transfusion.getName(), ConEnum.repeat24.getName()); // 输血重复开单
    }

    /**
     * 互斥开单项
     *
     * @param wordCrfDTO
     * @param ruleBaseDTO
     * @param billMsgList
     * @param ruleSimpleDTO
     * @param set 用于记录：A与B不宜同时进行, B与A不宜同时进行 只能提示一个
     */
    public void exclusionBill(WordCrfDTO wordCrfDTO, RuleBaseDTO ruleBaseDTO, List<BillMsg> billMsgList, String conType, RuleSimpleDTO ruleSimpleDTO, Set<String> set) {
        // 辅检开单项互斥
        List<Pacs> pacsOrder = wordCrfDTO.getPacsOrder();
        if (ListUtil.isNotEmpty(pacsOrder) && pacsOrder.size() > 1) {
            // 不同的辅检项目可能对应同一个uniqueName，提示显示多条
            Map<String, List<Pacs>> map = EntityUtil.makeEntityListMap(pacsOrder, "uniqueName");
            List<Pacs> orginNameList = map.get(ruleBaseDTO.getBaseLibName());
            if (map.get(ruleBaseDTO.getBaseLibName()) != null) {
                for (Pacs pacs : orginNameList) {
                    String orginName = pacs.getName();
                    // A与B不宜同时进行, B与A不宜同时进行 只能提示一个
                    if (!set.contains(ruleSimpleDTO.getLibName() + "******" + orginName)) {
                        set.add(ruleSimpleDTO.getLibName() + "******" + orginName);
                        set.add(orginName + "******" + ruleSimpleDTO.getLibName());
                        BillMsg commonBillMsg = MsgUtil.getCommonBillMsg(
                                ruleSimpleDTO.getInputName(), "",
                                orginName, conType, ruleSimpleDTO.getLibTypeName());
                        billMsgList.add(commonBillMsg);
                    }
                }
            }
        }
    }

    /**
     * 24小时重复开单入口
     *
     * @param billMsgList
     * @param itemList
     * @param type
     * @param conType 禁忌条件
     * @param <T>
     */
    public <T> void repeat24BillWithType(List<BillMsg> billMsgList, List<T> itemList, String type, String conType) {
        if (ListUtil.isEmpty(itemList)) {
            return;
        }
        // Map<String, List<T>> map = EntityUtil.makeEntityListMap(itemList, "name");
        Map<String, List<T>> map = CoreUtil.makeEntityListMap(itemList, "name", "uniqueName");
        for (String key : map.keySet()) {
            List<T> items = map.get(key);

            // 个数大于2个才比较
            if (ListUtil.isNotEmpty(items) && items.size() > 1) {
                // items 按照时间排序
                Collections.sort(items, new Comparator<T>() {
                    @Override
                    public int compare(T o1, T o2) {
                        String o1Str = (String) CoreUtil.getFieldValue(o1, "dateValue");
                        String o2Str = (String) CoreUtil.getFieldValue(o2, "dateValue");
                        if (StringUtil.isBlank(o1Str) || StringUtil.isBlank(o2Str)) {
                            return -1;
                        } else {
                            return o1Str.compareTo(o2Str);
                        }
                    }
                });

                // 比较时间是否在24小时内
                List<Date> dateList = new ArrayList<>();
                for (T it : items) {
                    String dateValue = (String) CoreUtil.getFieldValue(it, "dateValue");
                    if (StringUtil.isNotBlank(dateValue)) {
                        Date cur = CatalogueUtil.parseStringDate(dateValue);
                        // 如果为null，说明日期格式出错，不比较
                        if (cur != null) {
                            if (ListUtil.isNotEmpty(dateList)) {
                                Date last = dateList.get(dateList.size() - 1);
                                if (!CatalogueUtil.compareTime(last, cur, 60L * 24)) {
                                    String name = (String) CoreUtil.getFieldValue(it, "name");
                                    String uniqueName = (String) CoreUtil.getFieldValue(it, "uniqueName");
                                    BillMsg commonBillMsg = MsgUtil.getCommonBillMsg(
                                            name, uniqueName, name, ConEnum.repeat24.getName(), type);
                                    billMsgList.add(commonBillMsg);
                                    break;
                                } else {
                                    dateList.add(cur);
                                }
                            } else {
                                dateList.add(cur);
                            }
                        }
                    }
                }
            }
        }
    }

    /**
     * 高风险药品、手术
     *
     * @param highRiskNeoDTO
     * @param highRiskList
     */
    public void highRisk(HighRiskNeoDTO highRiskNeoDTO, List<BillMsg> highRiskList) {
        BillMsg billMsg = MsgUtil.getCommonHighRiskMsg(highRiskNeoDTO.getName(),
                highRiskNeoDTO.getHighLevel(), highRiskNeoDTO.getType());
        highRiskList.add(billMsg);
    }

    /**
     * 高风险药品、手术
     *
     * @param ruleSimpleDTO
     * @param highRiskList
     */
    public void highRisk(RuleSimpleDTO ruleSimpleDTO, List<BillMsg> highRiskList) {
        BillMsg billMsg = MsgUtil.getCommonHighRiskMsg(ruleSimpleDTO.getMsg(),
                ruleSimpleDTO.getInputName(), ruleSimpleDTO.getLibTypeName());
        highRiskList.add(billMsg);
    }

    /**
     * fa'zha
     *
     * @param highRiskNeoDTO
     * @param highRiskList
     */
    public void highRiskComplex(NodeNeoDTO nodeNeoDTO, List<BillMsg> highRiskList, HighRiskNeoDTO highRiskNeoDTO) {
        BillMsg billMsg = MsgUtil.getCommonHighRiskMsg(nodeNeoDTO.getVal(), highRiskNeoDTO.getName(), nodeNeoDTO.getTermtype());
        highRiskList.add(billMsg);
    }

    /**
     * 返回诊断数据源(只留阳性诊断)
     *
     * @param wordCrfDTO
     */
    public List<Item> getDiseaseSource(WordCrfDTO wordCrfDTO) {
        List<Item> diags = new ArrayList<>();
        CoreUtil.addList(diags, wordCrfDTO.getDiag()); // 结构化诊断
        CoreUtil.addAllConvert(diags, wordCrfDTO.getChiefLabel().getDiags());// 主诉诊断
        CoreUtil.addAllConvert(diags, wordCrfDTO.getPresentLabel().getDiags()); // 现病史诊断
        CoreUtil.addAllConvert(diags, wordCrfDTO.getPastLabel().getDiags()); // 既往史诊断
        return diags;
    }

    /**
     * 返回药品数据源
     *
     * @param wordCrfDTO
     */
    public List<Item> getDrugSource(WordCrfDTO wordCrfDTO) {
        List<Item> drugAll = new ArrayList<>();
        CoreUtil.addList(drugAll, wordCrfDTO.getDrug());
        CoreUtil.addAllConvert(drugAll, wordCrfDTO.getChiefLabel().getMedicines()); // 主诉药品
        CoreUtil.addAllConvert(drugAll, wordCrfDTO.getPresentLabel().getMedicines());// 现病史药品
        CoreUtil.addAllConvert(drugAll, wordCrfDTO.getPastLabel().getMedicines()); // 既往史药品
        return drugAll;
    }

    /**
     * 返回药品过敏数据源
     *
     * @param wordCrfDTO
     */
    public List<Item> getDrugAllergySource(WordCrfDTO wordCrfDTO) {
        List<Item> drugAllergyAll = new ArrayList<>();
        CoreUtil.addAllConvert(drugAllergyAll, wordCrfDTO.getPastLabel().getAllergyMedicines()); // 既往史过敏药品
        return drugAllergyAll;
    }

    public static void main(String[] args) {
        List<Pacs> pacsList = new ArrayList<>();
        Pacs pacs = new Pacs();
        pacs.setName("d1");
        pacs.setDateValue("2020-01-01");
        pacsList.add(pacs);

        Pacs pacs1 = new Pacs();
        pacs1.setName("d2");
        pacs1.setDateValue("2020-01-07");
        pacsList.add(pacs1);

        Pacs pacs3 = new Pacs();
        pacs3.setName("d3");
        pacs3.setDateValue("2020-01-09");
        pacsList.add(pacs3);
        CommonRule commonRule = new CommonRule();
        commonRule.sortByProperty(pacsList, "dateValue");
        for (Pacs bean : pacsList) {
            System.out.println(bean.getName());
        }


    }

}
