package com.diagbot.rule;

import com.diagbot.biz.push.entity.Lis;
import com.diagbot.dto.BillMsg;
import com.diagbot.dto.LisOtherDTO;
import com.diagbot.dto.RuleBaseDTO;
import com.diagbot.dto.RuleSimpleDTO;
import com.diagbot.dto.WordCrfDTO;
import com.diagbot.enums.LexiconEnum;
import com.diagbot.enums.TypeEnum;
import com.diagbot.util.CoreUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.MsgUtil;
import com.diagbot.util.StringUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;

/**
 * @description: 化验规则
 * @author: kwz
 * @time: 2020/8/3 14:47
 */
@Component
public class LisRule {

    @Autowired
    GroupRule groupRule;
    @Autowired
    CommonRule commonRule;

    /**
     * 比较化验——开单合理性
     *
     * @param inputLis
     * @param ruleBaseDTO
     * @param billMsgList
     * @param conType
     * @param ruleSimpleDTO
     */
    public void bill(List<Lis> inputLis, RuleBaseDTO ruleBaseDTO, List<BillMsg> billMsgList, String conType, RuleSimpleDTO ruleSimpleDTO) {
        if (ListUtil.isNotEmpty(inputLis) && StringUtil.isNotBlank(ruleBaseDTO.getBaseLibName())) {
            for (Lis lis : inputLis) {
                String uniqueName = lis.getUniqueName();
                if (StringUtil.isNotBlank(uniqueName) && uniqueName.equals(ruleBaseDTO.getBaseLibName())) {
                    Map<String, Object> map = CoreUtil.compareLis(ruleBaseDTO, lis);
                    if ((Boolean) map.get("flag") == true) {
                        BillMsg commonBillMsg = MsgUtil.getCommonBillMsg(ruleSimpleDTO.getInputName(),
                                ruleSimpleDTO.getLibName(), (String) map.get("msg"), conType, ruleSimpleDTO.getLibTypeName());
                        billMsgList.add(commonBillMsg);
                    }
                }
            }
        }
    }

    /**
     * 比较化验——高风险手术
     *
     * @param inputLis
     * @param ruleBaseDTO
     * @param highRiskList
     * @param ruleSimpleDTO
     */
    public void highOperation(List<Lis> inputLis, RuleBaseDTO ruleBaseDTO, List<BillMsg> highRiskList, RuleSimpleDTO ruleSimpleDTO) {
        if (ListUtil.isNotEmpty(inputLis) && StringUtil.isNotBlank(ruleBaseDTO.getBaseLibName())) {
            for (Lis lis : inputLis) {
                String uniqueName = lis.getUniqueName();
                if (StringUtil.isNotBlank(uniqueName) && uniqueName.equals(ruleBaseDTO.getBaseLibName())) {
                    Map<String, Object> map = CoreUtil.compareLis(ruleBaseDTO, lis);
                    if ((Boolean) map.get("flag") == true) {
                        BillMsg billMsg = MsgUtil.getCommonHighRiskMsg((String) map.get("msg"), ruleSimpleDTO.getInputName(), ruleSimpleDTO.getLibTypeName());
                        highRiskList.add(billMsg);
                    }
                }
            }
        }
    }

    /**
     * 比较化验——其他值提醒
     *
     * @param lisBean
     * @param ruleBaseDTOList
     * @param otherList
     * @param wordCrfDTO
     */
    public void otherLis(Lis lisBean, List<RuleBaseDTO> ruleBaseDTOList, List<BillMsg> otherList, WordCrfDTO wordCrfDTO) {
        if (ListUtil.isEmpty(ruleBaseDTOList)) {
            return ;
        }
        LisOtherDTO lisOtherDTO = new LisOtherDTO();
        int i = 0;
        for (RuleBaseDTO ruleBaseDTO : ruleBaseDTOList) {
            if (LexiconEnum.getEnum(ruleBaseDTO.getBaseLibType()) != null) { // 避免空指针
                Map<String, Object> map = null;
                switch (LexiconEnum.getEnum(ruleBaseDTO.getBaseLibType())) {
                    case LisSubName: // 化验细项
                        if (ruleBaseDTO.getBaseLibName().equals(lisBean.getUniqueName())) {
                            map = CoreUtil.compareLis(ruleBaseDTO, lisBean);
                            if ((Boolean) map.get("flag") == true) {
                                lisOtherDTO.setName((String) map.get("msg"));
                                i++;
                            }
                        }
                        break;
                    case Group: // 人群
                        map = groupRule.rule(wordCrfDTO, ruleBaseDTO);
                        if (CoreUtil.getMapFlag(map)) {
                            lisOtherDTO.getGroupMsg().add((String)map.get("msg"));
                            i++;
                        }
                        break;
                    case Disease: // 诊断
                        map = commonRule.compareItem(wordCrfDTO.getDiag(), ruleBaseDTO);
                        if (CoreUtil.getMapFlag(map)) {
                            lisOtherDTO.setDiseaseMsg((List<String>)map.get("msgList"));
                            i++;
                        }
                        break;
                    case Medicine:
                        map = commonRule.compareItem(wordCrfDTO.getDrug(), ruleBaseDTO);
                        if (CoreUtil.getMapFlag(map)) {
                            lisOtherDTO.setDrugMsg((List<String>)map.get("msgList"));
                            i++;
                        }
                        break;
                    default:
                        break;
                }
            }
        }
        // 条件全部符合
        if (i == ruleBaseDTOList.size()) {
            if (StringUtil.isNotBlank(lisOtherDTO.getName())) {
                // 诊断
                if (ListUtil.isNotEmpty(lisOtherDTO.getDiseaseMsg())) {
                    for (String str : lisOtherDTO.getDiseaseMsg()) {
                        BillMsg billMsg = MsgUtil.getCommonOtherMsg(TypeEnum.lis.getName(), lisOtherDTO.getName(), str, TypeEnum.disease.getName(), lisBean);
                        otherList.add(billMsg);
                    }
                }
                // 药品
                if (ListUtil.isNotEmpty(lisOtherDTO.getDrugMsg())) {
                    for (String str : lisOtherDTO.getDrugMsg()) {
                        BillMsg billMsg = MsgUtil.getCommonOtherMsg(TypeEnum.lis.getName(), lisOtherDTO.getName(), str, TypeEnum.drug.getName(), lisBean);
                        otherList.add(billMsg);
                    }
                }
                // 人群
                if (ListUtil.isNotEmpty(lisOtherDTO.getGroupMsg())) {
                    for (String str : lisOtherDTO.getGroupMsg()) {
                        BillMsg billMsg = MsgUtil.getCommonOtherMsg(TypeEnum.lis.getName(), lisOtherDTO.getName(), str, TypeEnum.group.getName(), lisBean);
                        otherList.add(billMsg);
                    }
                }

            }
        }
    }

    /**
     * 比较化验——其他值提醒
     *
     * @param lisBean
     * @param ruleBaseDTOList
     * @param otherList
     * @param wordCrfDTO
     */
    public void otherTransfusion(Lis lisBean, List<RuleBaseDTO> ruleBaseDTOList, String msg, List<BillMsg> otherList, WordCrfDTO wordCrfDTO) {
        if (ListUtil.isEmpty(ruleBaseDTOList)) {
            return ;
        }
        LisOtherDTO lisOtherDTO = new LisOtherDTO();
        lisOtherDTO.setMsg(msg);
        int i = 0;
        for (RuleBaseDTO ruleBaseDTO : ruleBaseDTOList) {
            if (LexiconEnum.getEnum(ruleBaseDTO.getBaseLibType()) != null) { // 避免空指针
                Map<String, Object> map = null;
                switch (LexiconEnum.getEnum(ruleBaseDTO.getBaseLibType())) {
                    case LisSubName: // 化验细项
                        if (ruleBaseDTO.getBaseLibName().equals(lisBean.getUniqueName())) {
                            map = CoreUtil.compareLis(ruleBaseDTO, lisBean);
                            if ((Boolean) map.get("flag") == true) {
                                lisOtherDTO.setName((String)map.get("msg"));
                                i++;
                            }
                        }
                        break;
                    case Disease: // 诊断
                        map = commonRule.compareItem(wordCrfDTO.getDiag(), ruleBaseDTO);
                        if (CoreUtil.getMapFlag(map)) {
                            lisOtherDTO.setDiseaseMsg((List<String>)map.get("msgList"));
                            i++;
                        }
                        break;
                    case Medicine: // 药品
                        map = commonRule.compareItem(wordCrfDTO.getDrug(), ruleBaseDTO);
                        if (CoreUtil.getMapFlag(map)) {
                            lisOtherDTO.setDrugMsg((List<String>)map.get("msgList"));
                            i++;
                        }
                    case Operation: // 手术
                        map = commonRule.compareItem(wordCrfDTO.getOperation(), ruleBaseDTO);
                        if (CoreUtil.getMapFlag(map)) {
                            lisOtherDTO.setOperationMsg((List<String>)map.get("msgList"));
                            i++;
                        }
                        break;
                    default:
                        break;
                }
            }
        }
        // 条件全部符合
        if (i == ruleBaseDTOList.size()) {
            if (StringUtil.isNotBlank(lisOtherDTO.getName())) {
                // 化验本身符合，其他不符号
                if (ListUtil.isEmpty(lisOtherDTO.getDiseaseMsg()) && ListUtil.isEmpty(lisOtherDTO.getOperationMsg())) {
                    BillMsg billMsg = MsgUtil.getCommonOtherMsg(TypeEnum.transfusion.getName(),
                            lisOtherDTO.getName(), msg, TypeEnum.lis.getName(), lisBean);
                    otherList.add(billMsg);
                } else {
                    if (ListUtil.isNotEmpty(lisOtherDTO.getDiseaseMsg())) {
                        for (String str : lisOtherDTO.getDiseaseMsg()) {
                            BillMsg billMsg = MsgUtil.getCommonOtherMsg(TypeEnum.transfusion.getName(),
                                    "诊断" + str + "，" + lisOtherDTO.getName(), msg, TypeEnum.disease.getName(), lisBean);
                            otherList.add(billMsg);
                        }
                    }
                    if (ListUtil.isNotEmpty(lisOtherDTO.getOperationMsg())) {
                        for (String str : lisOtherDTO.getOperationMsg()) {
                            BillMsg billMsg = MsgUtil.getCommonOtherMsg(TypeEnum.transfusion.getName(),
                                    lisOtherDTO.getName() + "，已开" + str, msg, TypeEnum.operation.getName(), lisBean);
                            otherList.add(billMsg);
                        }
                    }
                    // // 药品暂未用
                    // if (ListUtil.isNotEmpty(lisOtherDTO.getDrugMsg())) {
                    //     for (String str : lisOtherDTO.getDrugMsg()) {
                    //         BillMsg billMsg = MsgUtil.getCommonOtherMsg(TypeEnum.transfusion.getName(),
                    //                 lisOtherDTO.getName(), msg, TypeEnum.drug.getName(), lisBean);
                    //         otherList.add(billMsg);
                    //     }
                    // }
                }
            }
        }
    }

    /**
     * 比较化验——危急值（多个条件）
     *
     * @param wordCrfDTO
     * @param ruleBaseDTOList
     * @param criticalList
     */
    public void critical(Lis lisBean, List<RuleBaseDTO> ruleBaseDTOList, List<BillMsg> criticalList, WordCrfDTO wordCrfDTO) {
        if (ListUtil.isEmpty(ruleBaseDTOList)) {
            return ;
        }
        String msg = "";
        int i = 0;
        for (RuleBaseDTO ruleBaseDTO : ruleBaseDTOList) {
            if (LexiconEnum.getEnum(ruleBaseDTO.getBaseLibType()) != null) { // 避免空指针
                switch (LexiconEnum.getEnum(ruleBaseDTO.getBaseLibType())) {
                    case LisSubName: // 化验细项
                        if (ruleBaseDTO.getBaseLibName().equals(lisBean.getUniqueName())) {
                            Map<String, Object> map = CoreUtil.compareLis(ruleBaseDTO, lisBean);
                            if ((Boolean) map.get("flag") == true) {
                                i++;
                                msg = (String) map.get("msg");
                            }
                        }
                        break;
                    case Group: // 人群
                        Map map = groupRule.rule(wordCrfDTO, ruleBaseDTO);
                        if (CoreUtil.getMapFlag(map)) {
                            i++;
                        }
                        break;
                    default: break;
                }
            }
        }
        if (i == ruleBaseDTOList.size()) {
            BillMsg criticalMsg = MsgUtil.getCommonCriticalMsg(msg, TypeEnum.lis.getName());
            criticalList.add(criticalMsg);
        }
    }
}
