package com.diagbot.web;

import com.diagbot.dto.IndicationDTO;
import com.diagbot.dto.PushDTO;
import com.diagbot.dto.PushPlanDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.facade.IndicationFacade;
import com.diagbot.facade.PushFacade;
import com.diagbot.vo.IndicationPushVO;
import com.diagbot.vo.PushPlanVO;
import com.diagbot.vo.PushVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;

/**
 * @Description: CDSS核心API控制层
 * @author: zhoutg
 * @time: 2018/8/30 10:12
 */
@RestController
@RequestMapping("/core")
@Api(value = "CDSS核心API", tags = { "CDSS核心API" })
@SuppressWarnings("unchecked")
public class CoreController {

    @Autowired
    private IndicationFacade indicationFacade;
    @Autowired
    private PushFacade pushFacade;

    @ApiOperation(value = "开单合理性API[zhoutg]", notes = "ruleType（1:危急值提醒，2:开单合理项，3:高危药品、手术，4:其他提醒）")
    @PostMapping("/indication")
    public RespDTO<IndicationDTO> indication(@Valid @RequestBody IndicationPushVO indicationPushVO) {
        IndicationDTO indicationDTO = indicationFacade.indicationFac(indicationPushVO);
        return RespDTO.onSuc(indicationDTO);
    }

    @ApiOperation(value = "推送API[zhoutg]", notes = "featureType 类型(多选必填)，1：症状，4：查体结果，5：检验，6：检查，7：诊断，8：药品，9：手术，10：一般治疗")
    @PostMapping("/push")
    public RespDTO<PushDTO> push(@RequestBody PushVO pushVo) {
//        PushDTO pushDTO = pushFacade.pushFac(pushVo);
        PushDTO pushDTO1 = pushFacade.processAggreate(pushVo);
        return RespDTO.onSuc(pushDTO1);
    }

    @ApiOperation(value = "推送持续检验检查计划API[zhaops]", notes = "operationName：随访手术")
    @PostMapping("/pushPlan")
    public RespDTO<PushPlanDTO> pushPlan(@RequestBody PushPlanVO pushPlanVO) {
        PushPlanDTO pushPlanDTO = pushFacade.pushPlan(pushPlanVO);
        return RespDTO.onSuc(pushPlanDTO);
    }
}
